$ifndef _Graphic_Sub_
$define _Graphic_Sub_
'***************************************************************************
'***************************************************************************
'***************************************************************************
'***************************************************************************
'***************************************************************************
'*  Name     : Graphic_Sub_v0.x.inc                                              
'*  Founding Fathers : Gabi, VaGyver, Ray Ellam, Les Johnson                
'*  Version  : 0.2-6th September 2015                                                          
'*  Version  : 0.3-24th November 2016                                                          
'*  Version  : 0.4-13th September 2017                                      
'*           :     Proton FontCreator.exe fontdefinitions used              
'*           :     Display definition in hardware-part
'*           :     Calling method changed via precompiler                                                         
'*           :     Unused subroutines will not generate code                                                         
'*           :     Using the formulas/routines from Les Johnson in ILI9325.Inc (Bresenham)
'*           :          for DrawLine, DrawCircle (and FillCircle)
'*           :                                                              
'*           :                                                              
'***************************************************************************
GoTo OverGraphic_Sub
;
;   Graphic Subroutines
;
;***************************************************************************
;   Point Cursor and Print command in the right direction
;   Print AT and Cursor commands have a Xmax of a byte in GEN, Y is in WREG (and GEN2+GEN2H)
;----------------------------------------------------------------------------
Asm
GLCD@CUR
    Movwf Graphic_wCursorY,0                ;Value in W (Y.byte0) is also in GEN2
    Movff GEN2H,Graphic_wCursorYH           ;y could be greater than 255
    Movff GEN,Graphic_wCursorX              
    GoTo Graphic_GotoXYSub
Print
    GoTo Graphic_PrintCharSub
EndAsm
;----------------------------------------------------------------------------
;   Graphic_Init()
;Initialize all the Graphic stuff (if needed)
;Input : 
;Output:
;Notes :
;----------------------------------------------------------------------------
$define Graphic_Init()     '
        GoSub Graphic_InitSub
        
Graphic_InitSub:
    ;Until now nothing to Initialize 
Return
;----------------------------------------------------------------------------
;   (Graphic_)GotoXY(P1, P2)
;Administrate the Cursor position 
;Input : bCursorX bCursorY
;Output:
;Notes : Only $define needed without gosub but for the Debug there is a empty routine
;      : When removed dont forget to change GLCD@CUR (return instead goto)
;----------------------------------------------------------------------------
$define Graphic_GotoXY(P1, P2)  '
        Graphic_wCursorX = P1   '
        Graphic_wCursorY = P2   '
        GoSub Graphic_GotoXYSub
$define GotoXY(P1, P2)          '
        Graphic_wCursorX = P1   '
        Graphic_wCursorY = P2   '
        GoSub Graphic_GotoXYSub
        
Graphic_GotoXYSub:
Return
;----------------------------------------------------------------------------
;   (Graphic_)DrawPixel(P1, P2, P3)
;Gateway to underlaying Hardware Graphic Display Controller
;With other GDC only 
;Input : PixelX PixelY PixelColor
;Output:
;Notes :
;----------------------------------------------------------------------------
$define Graphic_DrawPixel(P1, P2, P3)   '
        Graphic_wDrawPixelX = P1        '
        Graphic_wDrawPixelY = P2        '
        Graphic_wDrawPixelColor = P3    '
        GoSub Graphic_DrawPixelSub
$define DrawPixel(P1, P2, P3)           '
        Graphic_wDrawPixelX = P1        '
        Graphic_wDrawPixelY = P2        '
        Graphic_wDrawPixelColor = P3    '
        GoSub Graphic_DrawPixelSub

Graphic_DrawPixelSub:
    ;Call the GraphicDisplayController Plot a Pixel routine. Gate to Hardware.
    ;Go to the underlaying hardware graphical display controller
Graphic_Plot(Graphic_wDrawPixelX, Graphic_wDrawPixelY, Graphic_wDrawPixelColor)
Return
;----------------------------------------------------------------------------
;   (Graphic_)DrawLine(P1, P2, P3, P4, P5)
;Draw a line This subroutine is "borrowed" from Les Johnson's ILI9325.inc
;Using the formulas from Bresenham. It uses only integer calculations
;See: www.tutorialspoint.com/computer graphics/line generation algorithm.html 
;Input : X0, Y0, X1, Y1, Color
;Output:
;Notes :
;----------------------------------------------------------------------------
$define Graphic_DrawLine(P1, P2, P3, P4, P5)     '
        Graphic_wDrawLineX0 = P1   '
        Graphic_wDrawLineY0 = P2   '
        Graphic_wDrawLineX1 = P3   '
        Graphic_wDrawLineY1 = P4   '
        Graphic_wColor = P5   '
        _Graphic_DrawLineSub
$define DrawLine(P1, P2, P3, P4, P5)     '
        Graphic_wDrawLineX0 = P1   '
        Graphic_wDrawLineY0 = P2   '
        Graphic_wDrawLineX1 = P3   '
        Graphic_wDrawLineY1 = P4   '
        Graphic_wColor = P5   '
        _Graphic_DrawLineSub
        
_Graphic_DrawLineSub Macro-
    GoSub Graphic_DrawLineSub
Endm

;#ifMacro- _Graphic_DrawLineSub        ;If not used no code 
Graphic_DrawLineSub:
        ;
        ;This is Les Johnson his GLCD_Line routine from ILI9325.inc 
        ;
    Graphic_wDeltaY = Graphic_wDrawLineY1 - Graphic_wDrawLineY0         ' Find the initial Y direction
    Graphic_wDeltaX = Graphic_wDrawLineX1 - Graphic_wDrawLineX0         ' Find the initial X direction

    Graphic_bStepY.SByte = 1                                            ' Default to a positive step for Ypos
    If Graphic_wDeltaY.15 = 1 Then                                      ' Check if negative
        Graphic_wDeltaY = -Graphic_wDeltaY                              ' Negate Graphic_wDeltaY
        Graphic_bStepY.SByte = -1                                       ' And indicate a negative step
    EndIf
    Graphic_bStepX.SByte = 1                                            ' Default to a positive step for Xpos
    If Graphic_wDeltaX.15 = 1 Then                                      ' Check if negative
        Graphic_wDeltaX = -Graphic_wDeltaX                              ' Negate Graphic_wDeltaX
        Graphic_bStepX.SByte = -1                                       ' And indicate a negative step
    EndIf
    Graphic_DrawPixel(Graphic_wDrawLineX0,Graphic_wDrawLineY0,Graphic_wColor)           ' The first pixel plot is a special case

    If Graphic_wDeltaX > Graphic_wDeltaY Then
        Graphic_wDLDecision.SWord = Graphic_wDeltaY - Graphic_wDeltaX
        While Graphic_wDrawLineX0 <> Graphic_wDrawLineX1
            If Graphic_wDLDecision.15 = 0 Then                            ' Check if positive
                Graphic_wDrawLineY0 = Graphic_wDrawLineY0 + Graphic_bStepY.SByte        ' Move on the Y axis directed by Graphic_bStepY
                Graphic_wDLDecision.SWord = Graphic_wDLDecision.SWord - Graphic_wDeltaX
            EndIf
            Graphic_wDrawLineX0 = Graphic_wDrawLineX0 + Graphic_bStepX.SByte            ' Move on the X axis directed by Graphic_bStepX
            Graphic_wDLDecision.SWord = Graphic_wDLDecision.SWord + Graphic_wDeltaY
            Graphic_DrawPixel(Graphic_wDrawLineX0,Graphic_wDrawLineY0,Graphic_wColor)   ' Draw or Erase the pixel at this position
        Wend
        GoTo Graphic_DrawLineEnd                                                          ' Return prematurely (this saves an else command)
    EndIf
    Graphic_wDLDecision.SWord = Graphic_wDeltaX - Graphic_wDeltaY
    
    While Graphic_wDrawLineY0 <> Graphic_wDrawLineY1
        If Graphic_wDLDecision.15 = 0 Then                                ' Check if positive
            Graphic_wDrawLineX0 = Graphic_wDrawLineX0 + Graphic_bStepX.SByte            ' Move on the X axis directed by Graphic_bStepX
            Graphic_wDLDecision.SWord = Graphic_wDLDecision.SWord - Graphic_wDeltaY
        EndIf
        Graphic_wDrawLineY0 = Graphic_wDrawLineY0 + Graphic_bStepY.SByte                ' Move on the Y axis directed by Graphic_bStepY
        Graphic_wDLDecision.SWord = Graphic_wDLDecision.SWord + Graphic_wDeltaX
        Graphic_DrawPixel(Graphic_wDrawLineX0,Graphic_wDrawLineY0,Graphic_wColor)       ' Draw or Erase the pixel at this position
    Wend    
Graphic_DrawLineEnd:
        ;save endpoint for DrawLinTo
    Graphic_wX1Save = Graphic_wDrawLineX1
    Graphic_wY1Save = Graphic_wDrawLineY1
    Return
;#endIfMacro-
;----------------------------------------------------------------------------
;   (Graphic_)DrawLinTo(P1, P2, P3)
; 
;Input : Graphic_bCommand
;Output:
;Notes :
;----------------------------------------------------------------------------
$define Graphic_DrawLinTo(P1, P2, P3)     '
        Graphic_wX1 = P1        '
        Graphic_wY1 = P2        '
        Graphic_wColor = P3     '
        _Graphic_DrawLinToSub
$define DrawLinTo(P1, P2, P3)   '
        Graphic_wX1 = P1        '
        Graphic_wY1 = P2        '
        Graphic_wColor = P3     '
        _Graphic_DrawLinToSub
        
_Graphic_DrawLinToSub Macro-
    GoSub Graphic_DrawLinToSub
Endm

;#ifMacro- _Graphic_DrawLinToSub        ;If not used no code 
Graphic_DrawLinToSub:
        ;Get old DrawLine endpoint as startingpoint (saved by DrawLine)
    Graphic_wX0 = Graphic_wX1Save
    Graphic_wY0 = Graphic_wY1Save
    Graphic_DrawLine(Graphic_wX0, Graphic_wY0, Graphic_wX1, Graphic_wY1, Graphic_wColor)
    Return
;#endIfMacro-
;----------------------------------------------------------------------------
;   (Graphic_)DrawRectangle(P1, P2, P3, P4, P5)
;Draw the borders of a rectangle 
;Input : X0, Y0, X1, Y1, Color
;Output:
;Notes :
;----------------------------------------------------------------------------
$define Graphic_DrawRectangle(P1, P2, P3, P4, P5)     '
        Graphic_wX0 = P1        '
        Graphic_wY0 = P2        '
        Graphic_wX1 = P3        '
        Graphic_wY1 = P4        '
        Graphic_wColor = P5     '
        _Graphic_DrawRectangleSub
$define DrawRectangle(P1, P2, P3, P4, P5)     '
        Graphic_wX0 = P1        '
        Graphic_wY0 = P2        '
        Graphic_wX1 = P3        '
        Graphic_wY1 = P4        '
        Graphic_wColor = P5     '
        _Graphic_DrawRectangleSub
        
_Graphic_DrawRectangleSub Macro-
    GoSub Graphic_DrawRectangleSub
Endm

;#ifMacro- _Graphic_DrawRectangleSub        ;If not used no code 
Graphic_DrawRectangleSub:
        ;Draw the 4 borderline of the rectangle
    Graphic_DrawLine(Graphic_wX0, Graphic_wY0, Graphic_wX0, Graphic_wY1, Graphic_wColor)
    Graphic_DrawLine(Graphic_wX0, Graphic_wY0, Graphic_wX1, Graphic_wY0, Graphic_wColor)
    Graphic_DrawLine(Graphic_wX1, Graphic_wY0, Graphic_wX1, Graphic_wY1, Graphic_wColor)
    Graphic_DrawLine(Graphic_wX0, Graphic_wY1, Graphic_wX1, Graphic_wY1, Graphic_wColor)
    Return
;#endIfMacro-
;----------------------------------------------------------------------------
;   (Graphic_)FillRectangle(P1, P2, P3, P4, P5)
;Fill a rectangle 
;Input : X0, Y0, X1, Y1, Color
;Output:
;Notes :
;----------------------------------------------------------------------------
$define Graphic_FillRectangle(P1, P2, P3, P4, P5)   '
        Graphic_wX0 = P1        '
        Graphic_wY0 = P2        '
        Graphic_wX1 = P3        '
        Graphic_wY1 = P4        '
        Graphic_wColor = P5     '
        _Graphic_FillRectangleSub
$define FillRectangle(P1, P2, P3, P4, P5)   '
        Graphic_wX0 = P1        '
        Graphic_wY0 = P2        '
        Graphic_wX1 = P3        '
        Graphic_wY1 = P4        '
        Graphic_wColor = P5     '
        _Graphic_FillRectangleSub
        
_Graphic_FillRectangleSub Macro-
    GoSub Graphic_FillRectangleSub
Endm

;#ifMacro- _Graphic_FillRectangleSub        ;If not used no code 
Graphic_FillRectangleSub:
        ;Fill a rectangle by means of lines
    For Graphic_wY0 = Graphic_wY0 To Graphic_wY1
        Graphic_DrawLine(Graphic_wX0, Graphic_wY0, Graphic_wX1, Graphic_wY0, Graphic_wColor)
    Next
    Return
;#endIfMacro-
;----------------------------------------------------------------------------
;   (Graphic_)DrawCircle(P1, P2, P3, P4)
;Draw a Circle This subroutine is "borrowed" from Les Johnson's ILI9325.inc
;Using the formulas from Bresenham. It uses only integer calculations
;See: www.tutorialspoint.com/computer graphics/circle generation algorithm.html 
;Input : Xm, Ym, Radius, Color
;Output:
;Notes :
;----------------------------------------------------------------------------
$define Graphic_DrawCircle(P1, P2, P3, P4)     '
        Graphic_wX0 = P1        '
        Graphic_wY0 = P2        '
        Graphic_wRadius = P3    '
        Graphic_wColor = P4     '
        _Graphic_DrawCircleSub
$define DrawCircle(P1, P2, P3, P4)     '
        Graphic_wX0 = P1        '
        Graphic_wY0 = P2        '
        Graphic_wRadius = P3    '
        Graphic_wColor = P4     '
        _Graphic_DrawCircleSub
        
_Graphic_DrawCircleSub Macro-
    GoSub Graphic_DrawCircleSub
Endm

;#ifMacro- _Graphic_DrawCircleSub        ;If not used no code 
Graphic_DrawCircleSub:
        ;
        ;This is Les Johnson his GLCD_Circle routine from ILI9325.inc
        ;
    If Graphic_wRadius = 0 Then Return                           ' Trap a radius of 0
    Graphic_wXYTemp = Graphic_wY0 - Graphic_wX0
    Graphic_wXTempP = 0
    Graphic_wYTempQ = Graphic_wRadius
    Graphic_wDCDecision = 3 - (2 * Graphic_wRadius)
    While Graphic_wXTempP <= Graphic_wYTempQ
        Graphic_wQuadA = Graphic_wX0 + Graphic_wXTempP
        Graphic_wQuadB = Graphic_wY0 + Graphic_wYTempQ
        Graphic_wQuadC = Graphic_wX0 - Graphic_wXTempP
        Graphic_wQuadD = Graphic_wY0 - Graphic_wYTempQ
        GoSub Graphic_DrawCirclePlot4
        Graphic_wQuadA = Graphic_wY0 + Graphic_wYTempQ - Graphic_wXYTemp
        Graphic_wQuadB = Graphic_wY0 + Graphic_wXTempP
        Graphic_wQuadC = Graphic_wY0 - Graphic_wYTempQ - Graphic_wXYTemp
        Graphic_wQuadD = Graphic_wY0 - Graphic_wXTempP
        GoSub Graphic_DrawCirclePlot4
        If Graphic_wDCDecision.15 = 1 Then                        ;negative?
            Graphic_wDCDecision = Graphic_wDCDecision + 6
            Graphic_wDCDecision = Graphic_wDCDecision + (4 * Graphic_wXTempP)
        Else
            Graphic_wDCDecision = Graphic_wDCDecision + 10
            Graphic_wDCDecision = Graphic_wDCDecision + (4 * (Graphic_wXTempP - Graphic_wYTempQ))
            Dec Graphic_wYTempQ
        EndIf
        Inc Graphic_wXTempP
    Wend
    Return
;
;Plot 4 locations simultaneously
;This is part of the Graphic_DrawCircleSub subroutine
;
Graphic_DrawCirclePlot4:
    Graphic_DrawPixel(Graphic_wQuadA, Graphic_wQuadB, Graphic_wColor)
    Graphic_DrawPixel(Graphic_wQuadA, Graphic_wQuadD, Graphic_wColor)
    Graphic_DrawPixel(Graphic_wQuadC, Graphic_wQuadB, Graphic_wColor)
    Graphic_DrawPixel(Graphic_wQuadC, Graphic_wQuadD, Graphic_wColor)
    Return
;#endIfMacro-
;----------------------------------------------------------------------------
;   (Graphic_)FillCircle(P1, P2, P3, P4)
;Fill a Circle This subroutine is "borrowed" from Les Johnson's ILI9325.inc
;Using the integer formulas from Bresenham. It uses only integer calculations
;See: www.tutorialspoint.com/computer graphics/circle generation algorithm.html 
;Input : Xm, Ym, Radius, Color
;Output:
;Notes :
;----------------------------------------------------------------------------
$define Graphic_FillCircle(P1, P2, P3, P4)     '
        Graphic_wX0 = P1        '
        Graphic_wY0 = P2        '
        Graphic_wRadius = P3    '
        Graphic_wColor = P4     '
        _Graphic_FillCircleSub
$define FillCircle(P1, P2, P3, P4)     '
        Graphic_wX0 = P1        '
        Graphic_wY0 = P2        '
        Graphic_wRadius = P3    '
        Graphic_wColor = P4     '
        _Graphic_FillCircleSub
        
_Graphic_FillCircleSub Macro-
    GoSub Graphic_FillCircleSub
Endm

;#ifMacro- _Graphic_FillCircleSub        ;If not used no code 
Graphic_FillCircleSub:
        ;
        ;This is Les Johnson his GLCD_Circle routine from ILI9325.inc
        ;The calculated points are use to draw lines to fill the circle
        ;
    If Graphic_wRadius = 0 Then Return                           ' Trap a radius of 0
    Graphic_wXYTemp = Graphic_wY0 - Graphic_wX0
    Graphic_wXTempP = 0
    Graphic_wYTempQ = Graphic_wRadius
    Graphic_wDCDecision = 3 - (2 * Graphic_wRadius)
    While Graphic_wXTempP <= Graphic_wYTempQ
        Graphic_wQuadA = Graphic_wX0 + Graphic_wXTempP
        Graphic_wQuadB = Graphic_wY0 + Graphic_wYTempQ
        Graphic_wQuadC = Graphic_wX0 - Graphic_wXTempP
        Graphic_wQuadD = Graphic_wY0 - Graphic_wYTempQ
            ;draw the 2 line to fill a part of the circle (middle-outside)
        Graphic_DrawLine(Graphic_wQuadA, Graphic_wQuadB, Graphic_wQuadA, Graphic_wQuadD, Graphic_wColor)
        Graphic_DrawLine(Graphic_wQuadC, Graphic_wQuadB, Graphic_wQuadC, Graphic_wQuadD, Graphic_wColor)
            ; next four circle points
        Graphic_wQuadA = Graphic_wY0 + Graphic_wYTempQ - Graphic_wXYTemp
        Graphic_wQuadB = Graphic_wY0 + Graphic_wXTempP
        Graphic_wQuadC = Graphic_wY0 - Graphic_wYTempQ - Graphic_wXYTemp
        Graphic_wQuadD = Graphic_wY0 - Graphic_wXTempP
            ;draw the 2 line to fill a part of the circle (outside-middle)
        Graphic_DrawLine(Graphic_wQuadA, Graphic_wQuadB, Graphic_wQuadA, Graphic_wQuadD, Graphic_wColor)
        Graphic_DrawLine(Graphic_wQuadC, Graphic_wQuadB, Graphic_wQuadC, Graphic_wQuadD, Graphic_wColor)
            ;decission time
        If Graphic_wDCDecision.15 = 1 Then                        ;negative?
            Graphic_wDCDecision = Graphic_wDCDecision + 6
            Graphic_wDCDecision = Graphic_wDCDecision + (4 * Graphic_wXTempP)
        Else
            Graphic_wDCDecision = Graphic_wDCDecision + 10
            Graphic_wDCDecision = Graphic_wDCDecision + (4 * (Graphic_wXTempP - Graphic_wYTempQ))
            Dec Graphic_wYTempQ
        EndIf
        Inc Graphic_wXTempP
    Wend
    Return
#endIfMacro-
;----------------------------------------------------------------------------
;   (Graphic_)SetFont(P1) or (Graphic_)SelectFont(P1)
;Administer the Font to be used 
;Input : Label of the FontTable
;Output:
;Notes :  PIC 18F dependency
;      :  The fonts are the result of the Proton FontConverter.exe
;----------------------------------------------------------------------------
$define Graphic_SetFont(P1)                 '
        Graphic_wFontAddress = AddressOf(P1) '
        _Graphic_SetFontSub
$define SetFont(P1)                 '
        Graphic_wFontAddress = AddressOf(P1) '
        _Graphic_SetFontSub
$define SelectFont(P1)                 '
        Graphic_wFontAddress = AddressOf(P1) '
        _Graphic_SetFontSub

_Graphic_SetFontSub Macro-
        GoSub Graphic_SetFontSub
Endm

#ifMacro- _Graphic_SetFontSub        ;If not used no code 
Graphic_SetFontSub:
        ;read FontHeight en #bytes per Pixelheight from fonttable (use 18F feature)
    Graphic_bFontHeight = cPtr8(Graphic_wFontAddress++) ;Read byte and increment address
    Graphic_bByteRead = cPtr8(Graphic_wFontAddress++) ;Read byte and increment address
        ;Now wFontAddress points to the addresstable in the Font    
        Return
#endIfMacro-
;----------------------------------------------------------------------------
;   Graphic_SetColor()
;Not Implemented 
;Input : 
;Output:
;Notes : 
;----------------------------------------------------------------------------
Graphic_SetColor:
Symbol  FontNormalColor = 1
Symbol  FontInverseColor = 0
Symbol  BackGroundNormalColor = 0
Symbol  BackGroundInverseColor = 1
Return    
;----------------------------------------------------------------------------
;   Graphic_PrintChar(P1)
;Print the Character in WREG = Proton Print command (or P1) 
;Input : WREG
;Output:
;Notes : PIC 18F dependency
;      : Uses the output from the "Proton FontConverter.exe" 
;----------------------------------------------------------------------------
$define Graphic_PrintChar(P1)       '
        WREG = P1                   '
        GoSub Graphic_PrintCharSub

Graphic_PrintCharSub:               ;ENTRYPOINT Proton Print 
    ;Save Character en Proton vars
Graphic_bSaveChar = WREG
Graphic_wSaveTblPtr = TBLPTRL
GEN4_temp = GEN4
GEN4H_temp = GEN4H
PP0_temp = PP0
PP0h_temp = PP0h
PP0hh_temp = PP0hh
PP0hhh_temp = PP0hhh
PP2_temp = PP2
PP2h_temp = PP2h
PP2hh_temp = PP2hh
PP2hhh_temp = PP2hhh
PP7_temp = PP7
PP7h_temp = PP7h
PP7hh_temp = PP7hh
PP7hhh_temp = PP7hhh
    ;Check if char in range 32-127
    ;First character is <Space> = 32
Graphic_bTemp = Graphic_bSaveChar - 32
If Graphic_bTemp > 95 Then GoTo Graphic_PrintCharEnd    ;Return to caller
    ;Get the address of the character image to print
    ;AddressTable is WordSize
Graphic_bTemp = Graphic_bTemp << 1                          ;Two bytes per characteraddress
Graphic_wCharAddress = Graphic_wFontAddress + Graphic_bTemp ;wCharAddress point to CharacterAddress in AddresTable
Graphic_wCharAddress = cPtr16(Graphic_wCharAddress)         ;now wCharAddress is relative start of character image address
Graphic_wCharAddress = Graphic_wFontAddress + Graphic_wCharAddress ;Absolute StartAddress of character image 
    ;Get the character width in pixels (first byte of the character image)
Graphic_bCharWidth =  cPtr8(Graphic_wCharAddress++)         ;inc wCharAddress automatic (18F feature)
    ;if the width = 0 there is no character to print so return to caller
If Graphic_bCharWidth = 0 Then GoTo Graphic_PrintCharEnd
    ;
    ;Print the character
    ;
Graphic_wPixelX = Graphic_wCursorX
Graphic_bWidthCount = 1
Repeat
        ;plot one vertical line of the Character image
    Graphic_wPixelY = Graphic_wCursorY
    Graphic_bHeightCount = 1
    Repeat
            ;Read a byte of fontdata
        Graphic_bFontByte = cPtr8(Graphic_wCharAddress++)
            ;Rotate the 8 bits right to plot each
        For Graphic_bTemp = 0 To 7                                       
                ;plot bit0
            If Graphic_bFontByte.0 = 1 Then
                    ;Inverse?
                If InverseFlag = 0 Then     ;No Inverse
                    Graphic_DrawPixel(Graphic_wPixelX, Graphic_wPixelY, FontNormalColor)
                Else                        ;Yes Inverse
                    Graphic_DrawPixel(Graphic_wPixelX, Graphic_wPixelY, FontInverseColor)
                EndIf
            Else        ;Graphic_bFontByte.0 = 0
                    ;do not print XOR background
                If XORflag = 0 Then         ;No XOR 
                        ;Inverse?
                    If InverseFlag = 0 Then     ;No Inverse
                        Graphic_DrawPixel(Graphic_wPixelX, Graphic_wPixelY, BackGroundNormalColor)
                    Else                        ;Yes Inverse
                        Graphic_DrawPixel(Graphic_wPixelX, Graphic_wPixelY, BackGroundInverseColor)
                    EndIf
                EndIf
            EndIf
                ;Follow the progress
            Inc Graphic_bHeightCount
                ;ready with this vertical line
            If Graphic_bHeightCount > Graphic_bFontHeight Then Break
                ;increment the Y-position for the next bit
            Inc Graphic_wPixelY 
                ;Rotate FontByte one position to the right for next bit
            Graphic_bFontByte = Graphic_bFontByte >> 1
        Next
    Until Graphic_bHeightCount > Graphic_bFontHeight
        ;next colomn of the character image
    Inc Graphic_wPixelX
    Inc Graphic_bWidthCount
Until Graphic_bWidthCount > Graphic_bCharWidth
    ;Character is printed 
    ;position Cursor for next Action
Graphic_wCursorX = Graphic_wPixelX
    ;Restore Proton vars
Graphic_PrintCharEnd:
GEN4 = GEN4_temp
GEN4H = GEN4H_temp
PP0 = PP0_temp
PP0h = PP0h_temp
PP0hh = PP0hh_temp
PP0hhh = PP0hhh_temp
PP2 = PP2_temp
PP2h = PP2h_temp
PP2hh = PP2hh_temp
PP2hhh = PP2hhh_temp
PP7 = PP7_temp
PP7h = PP7h_temp
PP7hh = PP7hh_temp
PP7hhh = PP7hhh_temp
TBLPTRL = Graphic_wSaveTblPtr.Word
WREG = Graphic_bSaveChar
Return
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
OverGraphic_Sub:
    ;Yes sneaky Init Graphic routines automatically
Graphic_Init()
;***************************************************************************
;   END Grapphic_Sub.Inc 
;***************************************************************************
$endif

