$ifndef _SSD1306_Sub_
$define _SSD1306_Sub_
'***************************************************************************
'***************************************************************************
'***************************************************************************
'***************************************************************************
'***************************************************************************
'*  Name     : SSD1306_Sub_v0.x.inc                                              
'*  Founding Fathers : Gabi, VaGyver, Ray Ellam, Les Johnson                
'*  Version  : 0.2-6th September 2015                                                          
'*  Version  : 0.3-24th November 2016                                                          
'*  Version  : 0.4-13th September 2017                                      
'*           :     SSD1306 hardware routines for 64x128 OLED display
'*           :     Display 32x128 added
'*           :     Calling method changed via precompiler                                                         
'*           :     Unused subroutines will not generate code                                                         
'*           :                                                              
'*           :                                                              
'*           :                                                              
'*           :                                                              
'***************************************************************************
GoTo OverSSD1306_Sub
;
;   Subroutines
;
;***************************************************************************
;   Point Cls in the right direction
;----------------------------------------------------------------------------
Asm
GLCD@CLS
    GoTo SSD1306_ClearScreenSub
EndAsm
;----------------------------------------------------------------------------
;   SSD1306_Init
;Init the SSD1306 display Controller
;Input : 
;Output:
;Notes :
;----------------------------------------------------------------------------
$define SSD1306_Init()         ' 
        GoSub SSD1306_InitSub
        
SSD1306_InitSub:
    ;Init the SSD1306 device
DelayMS 500                     'Wait a moment

    ;Sometimes the SSD1306 has cold start problems, Wake him up
HBStart
    HBusOut SSD1306
HBStop
DelayMS 10
      
HBStart                     ;Send a Start condition to the I2C bus
    HBusOut SSD1306         ;Slave Adress 
    
    HBusOut ControlByteCMD  ;
    HBusOut $AE             ;turn off oled panel
    HBusOut ControlByteCMD  ;
    HBusOut $D5             ;set display clock divide ratio/oscillator frequency
    HBusOut ControlByteCMD  ;
    HBusOut $F0             ;set divide ratio  default=$80
    HBusOut ControlByteCMD  ;
    HBusOut $A8             ;set multiplex ratio 1 to 64 
    HBusOut ControlByteCMD  ;
    HBusOut $3F             ;1/64 duty
    HBusOut ControlByteCMD  ;
    HBusOut $D3             ;set display offset
    HBusOut ControlByteCMD  ;
    HBusOut $00             ;not offset
    HBusOut ControlByteCMD  ;
    HBusOut $8d             ;set Charge Pump enable/disable
    HBusOut ControlByteCMD  ;
    HBusOut $14             ;set $10=disable 14=enable ???Why
    HBusOut ControlByteCMD  ;
    HBusOut $40             ;set start line address
    HBusOut ControlByteCMD  ;
    HBusOut $A6             ;set normal display
    HBusOut ControlByteCMD  ;
    HBusOut $A4             ;Disable Entire Display On (=RAM to Dispaly)
    HBusOut ControlByteCMD  ;
    HBusOut $A1             ;set segment re-map 127 to 0 =Reversed (Normal=$A0)
    HBusOut ControlByteCMD  ;
    HBusOut $C8             ;Set COM Output Scan Direction 63 to 0 =Reversed (Normal=$C0)
    HBusOut ControlByteCMD  ;
    HBusOut $DA             ;set com pins hardware configuration
    HBusOut ControlByteCMD  ;
    HBusOut $12             ;Disable COM remapping & Alternative COM pins
    HBusOut ControlByteCMD  ;
    HBusOut $81             ;set contrast control register
    HBusOut ControlByteCMD  ;
    HBusOut $FF             ;Default $8F $FF=Maximum $00=Minimum
    HBusOut ControlByteCMD  ;
    HBusOut $D9             ;set pre-charge period
    HBusOut ControlByteCMD  ;
    HBusOut $F1             ;Default $22 ?????
    HBusOut ControlByteCMD  ;
    HBusOut $DB             ;set vcomh
    HBusOut ControlByteCMD  ;
    HBusOut $40             ;Value WRONG Only Possible $00, $10, $20, $30
    HBusOut ControlByteCMD  ;
    HBusOut $AF             ;turn on oled panel
HBStop
    ;Zet de col en page positionering op 0,0
SSD1306_GotoStartPos:
HBStart
    HBusOut SSD1306         ;Slave address
    HBusOut ControlByteCMD  ;
    HBusOut $20             ;Set Memory Addressing Mode
    HBusOut ControlByteCMD  ;
    HBusOut $00             ;Horizontal Adressing ($01=Vertical $20=Paged)
    HBusOut ControlByteCMD  ;
    HBusOut $21             ;Set Colomn Address
    HBusOut ControlByteCMD  ;
    HBusOut $00             ;Start Colomn
    HBusOut ControlByteCMD  ;
    HBusOut (DisplayWidth - 1) ;End Colomn
    HBusOut ControlByteCMD  ;
    HBusOut $22             ;Set Page Address
    HBusOut ControlByteCMD  ;
    HBusOut $00             ;Start Page
    HBusOut ControlByteCMD  ;
    HBusOut (DisplayHeight / 8) - 1 ;End Page
HBStop    
Return
;----------------------------------------------------------------------------
;   SSD1306_Command(P1)
;Writes Controlbyte and SSD1306_bCommand to the SSD1306
;Input : SSD1306_bCommand
;Output:
;Notes :
;----------------------------------------------------------------------------
$define SSD1306_Command(P1)         '
        SSD1306_bCommand = P1       '
        GoSub SSD1306_CommandSub
        
SSD1306_CommandSub:
    ;Send ControlByteCommand($80), bCommand to SSD1306 device
    HBusOut SSD1306, [ControlByteCMD, SSD1306_bCommand]
Return
;----------------------------------------------------------------------------
;   SSD1306_Contrast(P1)
;Set the contrast value of the display 
;Input : ssd1306_bContrast
;Output:
;Notes :
;----------------------------------------------------------------------------
$define SSD1306_Contrast(P1)     '
        SSD1306_bContrast = P1   '
        _SSD1306_ContrastSub

_SSD1306_ContrastSub Macro-
    GoSub SSD1306_ContrastSub
Endm

#ifMacro- _SSD1306_ContrastSub          ;If not used no code        
SSD1306_ContrastSub:
    SSD1306_Command($81)                ;Send Contrast Command
    SSD1306_Command(SSD1306_bContrast)  ;Send Contrast value
    Return
#endIfMacro-
;----------------------------------------------------------------------------
;   SSD1306_DisplayOn()
;Switch the display ON 
;Input : 
;Output:
;Notes :
;----------------------------------------------------------------------------
$define SSD1306_DisplayOn()     '
        _SSD1306_DisplayOnSub

_SSD1306_DisplayOnSub Macro-
    GoSub SSD1306_DisplayOnSub
Endm

#ifMacro- _SSD1306_DisplayOnSub          ;If not used no code        
SSD1306_DisplayOnSub:
    SSD1306_Command($AF)                ;Send Display ON Command
    Return
#endIfMacro-
;----------------------------------------------------------------------------
;   SSD1306_DisplayOff()
;Switch the display Off 
;Input : 
;Output:
;Notes :
;----------------------------------------------------------------------------
$define SSD1306_DisplayOff()     '
        _SSD1306_DisplayOffSub

_SSD1306_DisplayOffSub Macro-
    GoSub SSD1306_DisplayOffSub
Endm

#ifMacro- _SSD1306_DisplayOffSub          ;If not used no code        
SSD1306_DisplayOffSub:
    SSD1306_Command($AE)                ;Send Display Off Command
    Return
#endIfMacro-
;----------------------------------------------------------------------------
;   SSD1306_DisplayInverse()
;Switch the display Inverse 
;Input : 
;Output:
;Notes :
;----------------------------------------------------------------------------
$define SSD1306_DisplayInverse()     '
        _SSD1306_DisplayInverseSub

_SSD1306_DisplayInverseSub Macro-
    GoSub SSD1306_DisplayInverseSub
Endm

#ifMacro- _SSD1306_DisplayInverseSub          ;If not used no code        
SSD1306_DisplayInverseSub:
    SSD1306_Command($A7)                ;Send Display Inverse Command
    Return
#endIfMacro-
;----------------------------------------------------------------------------
;   SSD1306_DisplayNormal()
;Switch the display Normal 
;Input : 
;Output:
;Notes :
;----------------------------------------------------------------------------
$define SSD1306_DisplayNormal()     '
        _SSD1306_DisplayNormalSub

_SSD1306_DisplayNormalSub Macro-
    GoSub SSD1306_DisplayNormalSub
Endm

#ifMacro- _SSD1306_DisplayNormalSub          ;If not used no code        
SSD1306_DisplayNormalSub:
    SSD1306_Command($A6)                ;Send Display Normal Command
    Return
#endIfMacro-
;----------------------------------------------------------------------------
;   SSD1306_ClearVideoMem()
;Clear the VideoMem array 
;Input : 
;Output:
;Notes :
;----------------------------------------------------------------------------
$define SSD1306_ClearVideoMem()     '
        _SSD1306_ClearVideoMemSub
$define ClearVideoMem()     '
        _SSD1306_ClearVideoMemSub
        
_SSD1306_ClearVideoMemSub Macro-
    GoSub SSD1306_ClearVideoMemSub
Endm

#ifMacro- _SSD1306_ClearVideoMemSub      ;If not used, don't compile        
SSD1306_ClearVideoMemSub:
    Clear bVideoMem
    Return
#endIfMacro-
;----------------------------------------------------------------------------
;   SSD1306_RefreshDisplay()
;Send the VideoMem to the SSD1306 Controller/Display
;Input : 
;Output:
;Notes :
;----------------------------------------------------------------------------
$define SSD1306_RefreshDisplay()     '
        GoSub SSD1306_RefreshDisplaySub
$define RefreshDisplay()     '
        GoSub SSD1306_RefreshDisplaySub

SSD1306_RefreshDisplaySub:              ;Always used no conditional compiling
    ;Put GDRAM pointer at 0,0  (for safety)
GoSub SSD1306_GotoStartPos    
    ;Send the bVideoMem array to the SSD1306
HBStart
    HBusOut SSD1306
    HBusOut ControlByteData
    Clear SSD1306_wTemp
    Repeat                      ;stream all the 1024 bytes to the SSD1306    
        HBusOut bVideoMem[SSD1306_wTemp]
        Inc SSD1306_wTemp
    Until SSD1306_wTemp = ArraySize
HBStop
Return
;----------------------------------------------------------------------------
;   SSD1306_ClearScreen()  = Cls
;Clear het SSD1306 display and the VideoMem 
;Input : 
;Output:
;Notes :
;----------------------------------------------------------------------------
$define SSD1306_ClearScreen()     '
        _SSD1306_ClearScreenSub

SSD1306_ClearScreenSub:
    SSD1306_ClearVideoMem()
    SSD1306_RefreshDisplay()
    Return
;----------------------------------------------------------------------------
;   Graphic_Plot(P1,P2,P3)
;****Gate to SSD1306 for graphic routines**** Plot a pixel in VideoMem 
;This routine is the ONLY entrance to the SSD1306 except some special direct SSD1306Commands
;like SSD1306_Contrast, SSD1306_Inverse and Normal, SSD1306_DisplayOff and DisplayOn 
;Input : bPlotX, bPlotY, bPlotColor
;Output:
;Notes : Calling routine uses Word variable, SSD1306 is max. 128 x 64 so byte size
;----------------------------------------------------------------------------
$define Graphic_Plot(P1, P2, P3)    '
        SSD1306_bPlotX = P1.Byte0         '
        SSD1306_bPlotY = P2.Byte0         '
        SSD1306_xPlotColor = P3.0   '
        GoSub SSD1306_PlotSub

SSD1306_PlotSub:
$ifdef SSD1306_DisplayHalf
        ;Half display only use the uneven bits of the SSD1306 VideoMemory in the y-axis
        ;So Ynew = (Y * 2) + 1
    SSD1306_bPlotY = (SSD1306_bPlotY << 1) + 1
$endif
    ;Within the Display bounderies? No then no action=return
If SSD1306_bPlotX >= DisplayWidth Then Return
If SSD1306_bPlotY >= DisplayHeight Then Return
    ;Get the VideoMem byte and bitposition in the byte of the pointed point
    ;8 yBits in one byte, so Y/8 Pages * displaywidth + xPixel = VideoMem position
wElement#No = ( SSD1306_bPlotY >> 3 ) * DisplayWidth + SSD1306_bPlotX
    ;Bit position of coordinate in bVideoByte = mod 8 bitposition
SSD1306_bPlotBitNum = SSD1306_bPlotY & %0000000000000111 
    ;Point FSRO to array VideoMem and add offset
FSR0 = AddressOf (bVideoMem) + wElement#No
    ;Get VideoMem byte
SSD1306_bPlotByte = INDF0                                  
    ;Put bColor in Byte
LoadBit SSD1306_bPlotByte, SSD1306_bPlotBitNum, SSD1306_xPlotColor
    ;Put Byte back in VideoMem
    ;for safety fill FSR0 again
FSR0 = AddressOf (bVideoMem) + wElement#No
INDF0 = SSD1306_bPlotByte                                  ;Get VideoMem byte
Return
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
OverSSD1306_Sub:
    ;Yes sneaky Init SSD1306 automatically
SSD1306_Init()
;***************************************************************************
;   END SSD1306_Sub.Inc 
;***************************************************************************
$endif

